# This is Open Source Software.
# Copyright 2000 S. Fermigier.
# This product includes software developed by Digital Creations
# for use in the Z Object Publishing Environment
# (http://www.zope.org/).

"""FileFolder object

FileFolders are like Folders, except that when you PUT something
in them (using either FTP ou WebDAV), that thing is creted as a File,
not a DTMLDocument.
"""

import Globals, Acquisition, OFS, webdav
from Globals import HTMLFile
import traceback

manage_addFileFolderForm = HTMLFile('fileFolderAdd', globals())

def manage_addFileFolder(self, id, title='', REQUEST=None):
	"""Add a new FileFolder object with id *id*."""

	ob = FileFolder()
	ob.id = str(id)
	ob.title = title
	self._setObject(id, ob)
	ob = self._getOb(id)

	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)


class FileFolder(OFS.Folder.Folder):
	"""
	FileFolders are like Folders, except that when you PUT something
	in them (using either FTP ou WebDAV), that thing is creted as a File,
	not a DTMLDocument.
	"""
	meta_type = 'FileFolder'

	def __getitem__(self, key):
		print '__getitem__', key
		v = self._getOb(key, None)
		if v is not None: return v
		if hasattr(self, 'REQUEST'):
			request = self.REQUEST
			method = request.get('REQUEST_METHOD', 'GET')
			if request.maybe_webdav_client and not method in ('GET', 'POST'):
				print 'creating NullResource'
				return NullResource(self, key).__of__(self)

		raise KeyError, key


class NullResource(webdav.NullResource.NullResource):
	"""NullResource with PUT method overriden."""

	def __init__(self, parent, name, request=None):
		self.__name__ = name
		self.__parent__ = parent
	   
	def PUT(self, REQUEST, RESPONSE):
		"""Create a new non-collection resource."""

		self.dav__init(REQUEST, RESPONSE)
		type = REQUEST.get_header('content-type', None)
		body = REQUEST.get('BODY', '')
		name = self.__name__

		if type is None:
			type, enc = OFS.content_types.guess_content_type(name, body)

		ob = OFS.Image.File(name, '', body, content_type=type)
		self.__parent__._setObject(name, ob)
		RESPONSE.setStatus(201)
		RESPONSE.setBody('')
		return RESPONSE

	def MKCOL(self, REQUEST, RESPONSE):
		"""Create a new collection resource."""

		try:
			print 'MKCOL'
			self.dav__init(REQUEST, RESPONSE)
			if REQUEST.get('BODY', ''):
				raise 'Unsupported Media Type', 'Unknown request body.'
			parent=self.__parent__
			name=self.__name__
			if hasattr(webdav.common.aq_base(parent), name):
				raise 'Method Not Allowed', 'The name %s is in use.' % name
			if not hasattr(parent, '__dav_collection__'):
				raise 'Forbidden', 'Cannot create collection at this location.'
			manage_addFileFolder(parent, name)
			RESPONSE.setStatus(201)
			RESPONSE.setBody('')
			return RESPONSE
		except:
			traceback.print_exc()

